#!/usr/bin/env python
# -*- coding: utf-8 -*-

# wlscan.py a frontend to iwlist to displays wireless networks in a
# more readable way.
# Copyright (C) 2009  Patrik Lembke
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

## setup
IWLIST = "sudo /sbin/iwlist wlan0 scan"

## do not edit below..
import os, sys
from math import ceil
from getopt import gnu_getopt

def get_aps():
    """Reads AP's etc and some info about them and return it"""
    iwlist = os.popen( IWLIST ).readlines()

    aps = dict() # will put other dicts inside this one
    
    for line in iwlist:
        if line.startswith( "          Cell" ):
            current = line
            aps[current] = { 'mac': line[ line.find( ':' ) +2 : -1 ] }
            
        elif line.startswith( "                    Channel" ):
            aps[current]['channel'] = int( line[ line.find( ':' ) +1 : -1 ].strip() )

        elif line.startswith( "                    Quality" ):
            aps[current]['quality'] = line[ line.find( '=' ) +1 :
                                                line.find( '  S' )]
            aps[current]['signal'] = line[ line.find( 'l=' ) +2 : -3 ]

        elif line.startswith( "                    ESSID" ):
            aps[current]['name'] = line[ line.find( '"' ) +1 : -2]

        elif line.startswith( "                    Encryption key" ):
            if line[-4:-1] == ":on":
                aps[current]['encryption'] = True
            else:
                aps[current]['encryption'] = False
    return aps

def open_closed_str( encrypted ):
    """Returns a much nicer then true false"""
    if not encrypted: return "open  "
    return "closed"

def quality_bar( quality_str ):
    """Transforms a quality string into a bar ex: [##...]"""
    cur, max = quality_str.split( '/' )
    cur, max = float( cur ), float( max )

    #print "%d (%d)" %( cur, int( floor( (cur/max) * 5 ) ) )
    
    ret = "#" * int( ceil( (cur/max) * 5 ) )
    ret += "." * ( 5 - len( ret ) )
    return "[" + ret + "]"
   
# --- main ---
# defaults
sorter = lambda x, y: x['channel'] - y['channel']
show_closed = True

# Option parser
opts, args = gnu_getopt( sys.argv[1:], 'oqh', [ 'open', 'quality', 'help' ] )

for opt, opt_arg in opts:
    if opt in ( '-o', '--open' ):
        show_closed = False
    elif opt in ( '-q', '--quality' ):
        sorter = lambda x, y: int( y['quality'].split( '/' )[0] ) - \
            int( x['quality'].split( '/' )[0] )
    elif opt in ( '-h', '--help' ):
        print "wlscan v0.0.2"
        print
        print "Usage:"
        print " -o\t--open\tOnly list open networks."
        print " -q\t--quality\tSort APs by quality (default by channel)."
        print " -h\t--help\tThis text."
        print
        exit( 0 )    

# real main
print "Channel Quality Signal\t\tOpen\t MAC\t\t   ESSID"
aps = get_aps().values()
aps.sort( sorter )

for ap in aps:
    if ap['encryption'] and not show_closed:
        pass
    else:
        print "%d\t%s\t%s \t%s   %s %s" %(
        ap['channel'], quality_bar( ap['quality'] ), ap['signal'],
        open_closed_str( ap['encryption' ] ),
        ap['mac'], ap['name'] )
